﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
public class GameManager : MonoBehaviour
{
    [SerializeField]
    private PlayerController _playerController;

    //Keeping track of all collected items
    public int collectedItems = 0;
    public Text _collItemsText;

    //Keeping track of current wave
    public int currentWave = 0;
    public GameObject[] spawnPoints;
    public GameObject enemyPrefab;

    private int _currentMobs = 0;


    public GameObject _pauseMenu;


    //Boss Stuff
    public GameObject _bossHealthSlider;
    [SerializeField]
    private GameObject _bossPrefab;
    [SerializeField]
    private GameObject _bossSpawned;

    public GameObject _bossSpawnPoint;

    public GameObject _winPanel;

    private bool _isInBossFight = false;


    //Attack speed
    public Text _attackSpeed;


    //Player Gold Coins
    public int goldCoins = 0;

    [SerializeField]
    private Text _coinsText;

    [SerializeField]
    private Text _currentWaveText;

    public bool _hasRepair = false;

    public void Start()
    {
        StartWave();
    }

    public void Update()
    {
        CurrentCollectedItems();
        BossFight();
        UpdateAttackSpeedUI();
        UpdateCoinsUI();
        UpdateWaveUI();
        Pause();
        Win();

    }

    public void CurrentCollectedItems()
    {
        _collItemsText.text = "Collected items : " + collectedItems.ToString() + "/6";
    }

    public void StartWave()
    {
        int mobsToSpawn = 4 + currentWave;
        for (int i = 0; i < mobsToSpawn; i++)
        {
            int randomSpawnPoint = Random.Range(0, spawnPoints.Length);
            StartCoroutine(spawnEnemy(randomSpawnPoint));
        }

    }

    public void UpdateAttackSpeedUI()
    {
        _attackSpeed.text = ((_playerController.attackSpeed)/100).ToString();
    }



    private IEnumerator spawnEnemy(int randSP)
    {

        yield return new WaitForSeconds(4f);
        GameObject _go = GameObject.Instantiate(enemyPrefab, spawnPoints[randSP].transform.position, Quaternion.identity);
        _currentMobs++;

    }

    public void EnemyKilled()
    {
        _currentMobs -= 1;
        EndWave();
    }

    private void EndWave()
    {
        if (_currentMobs <= 0)
        {currentWave++;
            if (currentWave < 2)
            {                
                StartWave();
            }

            if (currentWave == 2)
            {
                StartBossFight();
            }

        }


    }

    IEnumerator BossStartRoutine()
    {
        yield return new WaitForSeconds(3f);

        GetComponent<AudioManager>().ChangeAudioClip(2);
        _bossHealthSlider.SetActive(true);
        GameObject _go = GameObject.Instantiate(_bossPrefab, _bossSpawnPoint.transform.position, Quaternion.identity);
        _bossSpawned = _go;
        _bossHealthSlider.GetComponent<Slider>().value = _go.GetComponent<BossAI>()._health;

        _isInBossFight = true;
    }


    //Boss Mode
    private void StartBossFight()
    {
        StartCoroutine(BossStartRoutine());
        
        
    }

    private void BossFight()
    {
        if (_isInBossFight)
        {
            _bossHealthSlider.GetComponent<Slider>().value = _bossSpawned.GetComponent<BossAI>()._health;
        }
        
    }

    private void UpdateCoinsUI()
    {
        _coinsText.text = goldCoins.ToString();
    }

    private void UpdateWaveUI()
    {
        _currentWaveText.text = "Current Wave: " + currentWave.ToString() + " / " + 2.ToString();
    }
    public void BossDied()
    {
        _bossHealthSlider.SetActive(false);
        _isInBossFight = false;
    }

    public void Win()
    {
        if (_hasRepair)
        {
            GetComponent<SceneLoader>().EndGame();
            Time.timeScale = 0;
        }
    }

    public void Pause()
    {
        if (Input.GetKeyDown(KeyCode.Escape))
        {
            _pauseMenu.SetActive(true);
        }
    }
}
