﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Audio;

public class PlayerController : MonoBehaviour
{
    public AudioSource yeet, hurt;

    [SerializeField]
    private Rigidbody2D _playerRB;

    [SerializeField]
    private HeartManager _heartManager;

    [SerializeField]
    private GameManager _gm;

    public GameObject deathScreen;

    private float horizontal;
    private float vertical;

    private enum _lookDir {empty,left,right,up,down};

    private _lookDir _currentLookDir = _lookDir.empty;

    public float movementSpeed = 5.0f;

    public float attackSpeed = 750f;

    public GameObject shootPoint;

    //Get the ammo prefab
    public GameObject _wrench;
    private float _currentShootTime = 0.0f;


    public int _currentHealth = 75;
    public int _maxHealth = 75;

    //Inventory
    public int _RepairItemID = -1;

    private void Start()
    {
        _gm = GameObject.FindGameObjectWithTag("GameController").GetComponent<GameManager>();
    }

    private void Update()
    {
        InputManager();
        FlipSides();
        CheckIfHasRepair();
        Death();
    }
    
    private void FixedUpdate()
    {
        PlayerMove();
    }

    private void InputManager()
    {
        horizontal = Input.GetAxis("Horizontal")*movementSpeed;
        vertical = Input.GetAxis("Vertical")*movementSpeed;

        if (horizontal == 0 && vertical == 0)
        {
            GetComponent<Animator>().SetBool("isWalking", false);
        }
        else { GetComponent<Animator>().SetBool("isWalking", true); }

        if (Input.GetKey(KeyCode.RightArrow))
        {
            _currentLookDir = _lookDir.right;
            ShootWrench();
            yeet = GetComponent<AudioSource>();
            yeet.Play();
        }
        else if (Input.GetKey(KeyCode.LeftArrow))
        {
            _currentLookDir = _lookDir.left;
            ShootWrench();
        }
        else if (Input.GetKey(KeyCode.UpArrow))
        {
            _currentLookDir = _lookDir.up;
            ShootWrench();
        }
        else if (Input.GetKey(KeyCode.DownArrow))
        {
            _currentLookDir = _lookDir.down;
            ShootWrench();
        }
        else { _currentLookDir = _lookDir.empty; }

    }

    private void PlayerMove()
    {
        Vector2 moveVector = new Vector2(horizontal, vertical);

        _playerRB.velocity = moveVector;
        

    }
    private void FlipSides()
    {
        if (Input.GetAxis("Horizontal") < 0)
        {
            GetComponent<SpriteRenderer>().flipX = true;
        }
        else if (Input.GetAxis("Horizontal") > 0) { GetComponent<SpriteRenderer>().flipX = false; }
    }

    private void ShootWrench()
    {


        if (Time.time >= _currentShootTime)
        {
            yeet = GetComponent<AudioSource>();
            yeet.Play();            
            //GetComponent<Animator>().SetTrigger("Attack");
            GameObject _shot = GameObject.Instantiate(_wrench, shootPoint.transform.position, Quaternion.identity);
            Vector2 _forceVector = new Vector2(0,0);
            if (_currentLookDir == _lookDir.left)
            {
                _forceVector = new Vector2(-attackSpeed, 0);
            }
            else if (_currentLookDir == _lookDir.right)
            {
                _forceVector = new Vector2(attackSpeed, 0);
            }
            else if (_currentLookDir == _lookDir.up)
            {
                _forceVector = new Vector2(0, attackSpeed);
            }
            else if(_currentLookDir == _lookDir.down)
            {
                _forceVector = new Vector2(0, -attackSpeed);
            }
            else { _forceVector = new Vector2(0, 0); }

            _shot.GetComponent<Rigidbody2D>().AddForce(_forceVector);
            _currentShootTime =Time.time + 0.5f;
        }
    }

    private void Death()
    {
        if (_currentHealth <= 0)
        {
            deathScreen.SetActive(true);
            Time.timeScale = 0.0f;
            Debug.Log("Dead!");
        }
    }

    public void Heal(int hpToRestore)
    {
        _currentHealth += hpToRestore;

    }

    public void UpgradeHealth()
    {
        _maxHealth += 25;
        _heartManager._upgrades += 1;
        Heal(25);
    }

    public void CheckIfHasRepair()
    {
        if (_RepairItemID == 0)
        {
            _gm._hasRepair = true;
        }
    }
}
